
#include "PipelineBindings.hpp"
#include "NodeBindings.hpp"

// depthai
#include "depthai/pipeline/Pipeline.hpp"

// depthai - nodes
#include "depthai/pipeline/node/XLinkIn.hpp"
#include "depthai/pipeline/node/XLinkOut.hpp"
#include "depthai/pipeline/node/NeuralNetwork.hpp"
#include "depthai/pipeline/node/ColorCamera.hpp"
#include "depthai/pipeline/node/VideoEncoder.hpp"
#include "depthai/pipeline/node/SPIOut.hpp"
#include "depthai/pipeline/node/ImageManip.hpp"
#include "depthai/pipeline/node/MonoCamera.hpp"
#include "depthai/pipeline/node/StereoDepth.hpp"
#include "depthai/pipeline/node/DetectionNetwork.hpp"
#include "depthai/pipeline/node/Script.hpp"
#include "depthai/pipeline/node/SystemLogger.hpp"
#include "depthai/pipeline/node/SpatialLocationCalculator.hpp"
#include "depthai/pipeline/node/SpatialDetectionNetwork.hpp"

// depthai-shared
#include "depthai-shared/properties/GlobalProperties.hpp"



std::shared_ptr<dai::Node> createNode(dai::Pipeline& p, py::object class_){
    auto nodeCreateMap = NodeBindings::getNodeCreateMap();
    for(auto& kv : nodeCreateMap){
        auto& node = kv.first;
        auto& create = kv.second;
        if(node.is(class_)){
            return create(p, class_);
        }
    }
    return nullptr;
}

void PipelineBindings::bind(pybind11::module& m){

    using namespace dai;


    // Bind global properties
    py::class_<GlobalProperties>(m, "GlobalProperties", DOC(dai, GlobalProperties))
        .def_readwrite("leonOsFrequencyHz", &GlobalProperties::leonCssFrequencyHz)
        .def_readwrite("leonRtFrequencyHz", &GlobalProperties::leonMssFrequencyHz)
        .def_readwrite("pipelineName", &GlobalProperties::pipelineName)
        .def_readwrite("pipelineVersion", &GlobalProperties::pipelineVersion)
        ;



    // bind pipeline
    py::class_<Pipeline>(m, "Pipeline", DOC(dai, Pipeline, 2))
        .def(py::init<>(), DOC(dai, Pipeline, Pipeline))
        //.def(py::init<const Pipeline&>())
        .def("getGlobalProperties", &Pipeline::getGlobalProperties, DOC(dai, Pipeline, getGlobalProperties))
        //.def("create", &Pipeline::create<node::XLinkIn>)
        .def("remove", &Pipeline::remove, py::arg("node"), DOC(dai, Pipeline, remove))
        .def("getAllNodes", static_cast<std::vector<std::shared_ptr<const Node>> (Pipeline::*)() const>(&Pipeline::getAllNodes), py::return_value_policy::reference_internal, DOC(dai, Pipeline, getAllNodes))
        .def("getAllNodes", static_cast<std::vector<std::shared_ptr< Node>> (Pipeline::*)()>(&Pipeline::getAllNodes), py::return_value_policy::reference_internal, DOC(dai, Pipeline, getAllNodes))
        .def("getNode", static_cast<std::shared_ptr<const Node> (Pipeline::*)(Node::Id) const>(&Pipeline::getNode), py::return_value_policy::reference_internal, DOC(dai, Pipeline, getNode))
        .def("getNode", static_cast<std::shared_ptr<Node> (Pipeline::*)(Node::Id)>(&Pipeline::getNode), py::return_value_policy::reference_internal, DOC(dai, Pipeline, getNode))
        .def("getConnections", &Pipeline::getConnections, DOC(dai, Pipeline, getConnections), DOC(dai, Pipeline, getConnections))
        .def("getConnectionMap", &Pipeline::getConnectionMap, DOC(dai, Pipeline, getConnectionMap), py::return_value_policy::reference_internal, DOC(dai, Pipeline, getConnectionMap))
        .def("getNodeMap", &Pipeline::getNodeMap, DOC(dai, Pipeline, getNodeMap), py::return_value_policy::reference_internal, DOC(dai, Pipeline, getNodeMap))
        .def("link", &Pipeline::link, DOC(dai, Pipeline, link), DOC(dai, Pipeline, link))
        .def("unlink", &Pipeline::unlink, DOC(dai, Pipeline, unlink), DOC(dai, Pipeline, unlink))
        .def("getAssetManager", static_cast<const AssetManager& (Pipeline::*)() const>(&Pipeline::getAssetManager), py::return_value_policy::reference_internal, DOC(dai, Pipeline, getAssetManager))
        .def("getAssetManager", static_cast<AssetManager& (Pipeline::*)()>(&Pipeline::getAssetManager), py::return_value_policy::reference_internal, DOC(dai, Pipeline, getAssetManager))
        .def("setOpenVINOVersion", &Pipeline::setOpenVINOVersion, py::arg("version") = Pipeline::DEFAULT_OPENVINO_VERSION, DOC(dai, Pipeline, setOpenVINOVersion), DOC(dai, Pipeline, setOpenVINOVersion))

        // 'Template' create function
        .def("create", [](dai::Pipeline& p, py::object class_) {
            auto node = createNode(p, class_);
            if(node == nullptr){
                throw std::invalid_argument(std::string(py::str(class_)) + " is not a subclass of depthai.node");
            }
            return node;
        })

        // DEPRECATED, use pipeline.create([class name])
        .def("createXLinkIn", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::XLinkIn>();
        })
        .def("createXLinkOut", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::XLinkOut>();
        })
        .def("createNeuralNetwork", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::NeuralNetwork>();
        })
        .def("createColorCamera", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::ColorCamera>();
        })
        .def("createVideoEncoder", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::VideoEncoder>();
        })
        .def("createSPIOut", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::SPIOut>();
        })
        .def("createImageManip", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::ImageManip>();
        })
        .def("createMonoCamera", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::MonoCamera>();
        })
        .def("createStereoDepth", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::StereoDepth>();
        })
        .def("createMobileNetDetectionNetwork", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::MobileNetDetectionNetwork>();
        })
        .def("createYoloDetectionNetwork", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::YoloDetectionNetwork>();
        })
        .def("createSystemLogger", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::SystemLogger>();
        })
        .def("createSpatialLocationCalculator", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::SpatialLocationCalculator>();
        })
        .def("createMobileNetSpatialDetectionNetwork", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::MobileNetSpatialDetectionNetwork>();
        })
        .def("createYoloSpatialDetectionNetwork", [](Pipeline& p){
            PyErr_WarnEx(PyExc_DeprecationWarning, "create[Node]() is deprecated, use create([Node]) instead.", 1);
            return p.create<node::YoloSpatialDetectionNetwork>();
        })
        ;
    
}
